@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2010 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -493,7 +493,8 @@ public class ClassSpecificationVisitorFactory
     private static boolean containsWildCards(String string)
     {
         return string != null &&
-            (string.indexOf('*')   >= 0 ||
+            (string.indexOf('!')   >= 0 ||
+             string.indexOf('*')   >= 0 ||
              string.indexOf('?')   >= 0 ||
              string.indexOf('%')   >= 0 ||
              string.indexOf(',')   >= 0 ||
