@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2010 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -139,7 +139,9 @@ implements   MemberVisitor,
     {
         if (!KeepMarker.isKept(programClass))
         {
-            notePrinter.print("Note: the configuration keeps the entry point '" +
+            notePrinter.print(referencingClass.getName(),
+                              programClass.getName(),
+                              "Note: the configuration keeps the entry point '" +
                               ClassUtil.externalClassName(referencingClass.getName()) +
                               " { " +
                               (isField ?
