@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2010 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -28,7 +28,7 @@ import proguard.classfile.visitor.ClassVisitor;
  *
  * @author Eric Lafortune
  */
-public class LocalVariableInfo
+public class LocalVariableInfo implements Comparable
 {
     public int u2startPC;
     public int u2length;
@@ -80,4 +80,20 @@ public class LocalVariableInfo
             referencedClass.accept(classVisitor);
         }
     }
+
+
+    // Implementations for Comparable.
+
+    public int compareTo(Object object)
+    {
+        LocalVariableInfo other = (LocalVariableInfo)object;
+
+        return
+            this.u2startPC          < other.u2startPC          ? -1 : this.u2startPC          > other.u2startPC          ? 1 :
+            this.u2length           < other.u2length           ? -1 : this.u2length           > other.u2length           ? 1 :
+            this.u2index            < other.u2index            ? -1 : this.u2index            > other.u2index            ? 1 :
+            this.u2descriptorIndex  < other.u2descriptorIndex  ? -1 : this.u2descriptorIndex  > other.u2descriptorIndex  ? 1 :
+            this.u2nameIndex        < other.u2nameIndex        ? -1 : this.u2nameIndex        > other.u2nameIndex        ? 1 :
+                                                                                                                           0;
+    }
 }
