@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2010 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -690,7 +690,7 @@ public abstract class Instruction
     /**
      * Writes the Instruction at the given offset in the given code array.
      */
-    public final void write(byte[] code, int offset)
+    public void write(byte[] code, int offset)
     {
         // Write the wide opcode, if necessary.
         if (isWide())
