@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2010 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -278,9 +278,7 @@ implements   ClassVisitor,
         public void visitUnknownAttribute(Clazz clazz, UnknownAttribute unknownAttribute)
         {
             // Write the unknown information.
-            byte[] info = new byte[unknownAttribute.u4attributeLength];
-            dataOutput.write(info);
-            unknownAttribute.info = info;
+            dataOutput.write(unknownAttribute.info);
         }
 
 
