@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2010 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -432,7 +432,7 @@ class ClassPathPanel extends ListPanel
                     filter = new StringBuffer().append('(');
                 }
 
-                filter.append(ListUtil.commaSeparatedString(additionalFilter));
+                filter.append(ListUtil.commaSeparatedString(additionalFilter, true));
             }
 
             return filter;
