@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2010 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -22,6 +22,7 @@ package proguard.gui;
 
 import javax.swing.*;
 import java.awt.*;
+import java.lang.reflect.InvocationTargetException;
 
 
 /**
@@ -46,10 +47,17 @@ final class MessageDialogRunnable implements Runnable
                                          String    title,
                                          int       messageType)
     {
-        SwingUtil.invokeAndWait(new MessageDialogRunnable(parentComponent,
-                                                          message,
-                                                          title,
-                                                          messageType));
+        try
+        {
+            SwingUtil.invokeAndWait(new MessageDialogRunnable(parentComponent,
+                                                              message,
+                                                              title,
+                                                              messageType));
+        }
+        catch (Exception e)
+        {
+            // Nothing.
+        }
     }
 
 
