@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2010 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -57,7 +57,11 @@ public class FileDataEntry implements DataEntry
     }
 
 
-    // Implementations for InputEntry.
+    public boolean isDirectory()
+    {
+        return file.isDirectory();
+    }
+
 
     public InputStream getInputStream() throws IOException
     {
