@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2010 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -35,9 +35,9 @@ import proguard.evaluation.value.*;
 public class LoadingInvocationUnit
 extends      BasicInvocationUnit
 {
-    private boolean loadFieldValues;
-    private boolean loadMethodParameterValues;
-    private boolean loadMethodReturnValues;
+    private final boolean loadFieldValues;
+    private final boolean loadMethodParameterValues;
+    private final boolean loadMethodReturnValues;
 
 
     /**
@@ -84,8 +84,6 @@ extends      BasicInvocationUnit
                     value.isParticular())
                 {
                     return value;
-//                // Make sure the value is refreshed.
-//                return refresh(value);
                 }
             }
         }
@@ -110,8 +108,6 @@ extends      BasicInvocationUnit
                     value.isParticular())
                 {
                     return value;
-//                // Make sure the value is refreshed.
-//                return refresh(value);
                 }
             }
         }
@@ -134,8 +130,6 @@ extends      BasicInvocationUnit
                 value.isParticular())
             {
                 return value;
-//            // Make sure the value is refreshed.
-//            return refresh(value);
             }
         }
 
@@ -163,8 +157,6 @@ extends      BasicInvocationUnit
                     value.isParticular())
                 {
                     return value;
-//                // Make sure the value is refreshed.
-//                return refresh(value);
                 }
             }
         }
@@ -173,8 +165,8 @@ extends      BasicInvocationUnit
                                           refConstant,
                                           type);
     }
-//
-//
+
+
 //    // Small utility methods.
 //
 //    private Value refresh(Value value)
